@@ -1,7 +1,7 @@
 import math
 from cStringIO import StringIO as sio
 
-from twisted.internet import error, defer
+from twisted.internet import error, defer, reactor
 from twisted.python import failure, reflect
 from twisted.trial import unittest
 from twisted.protocols import amp
@@ -121,6 +121,16 @@ class BadChild(child.AMPChild):
     Die.responder(die)
 
 class TestAMPConnector(unittest.TestCase):
+    def setUp(self):
+        """
+        The only reason why this method exists is to let 'trial ampoule'
+        to install the signal handlers (#3178 for reference).
+        """
+        super(TestAMPConnector, self).setUp()
+        d = defer.Deferred()
+        reactor.callLater(0, d.callback, None)
+        return d
+
     def _makeConnector(self, s, sa):
         a = FakeAMP(sa)
         ac = main.AMPConnector(a)
@@ -663,3 +673,35 @@ class TestProcessPool(unittest.TestCase):
             ).addCallback(_work
             ).addCallback(_check
             ).addCallback(lambda _: pp.stop())
+
+    def test_processTimeout(self):
+        """
+        Test that a call that doesn't finish within the given timeout
+        time is correctly handled.
+        """
+        pp = pool.ProcessPool(WaitingChild, min=1, max=1)
+        
+        def _work(_):
+            d = pp.callRemote(First, data="ciao", _timeout=1)
+            self.assertFailure(d, error.ProcessTerminated)
+            return d
+
+        return pp.start(
+            ).addCallback(_work
+            ).addCallback(lambda _: pp.stop())
+
+    def test_processGlobalTimeout(self):
+        """
+        Test that a call that doesn't finish within the given global
+        timeout time is correctly handled.
+        """
+        pp = pool.ProcessPool(WaitingChild, min=1, max=1, timeout=1)
+        
+        def _work(_):
+            d = pp.callRemote(First, data="ciao")
+            self.assertFailure(d, error.ProcessTerminated)
+            return d
+
+        return pp.start(
+            ).addCallback(_work
+            ).addCallback(lambda _: pp.stop())
